@@ -49,16 +49,23 @@ proc selfupdate::main {{optionslist {}} {updatestatusvar {}}} {
         set updatestatus no
     }
 
+    set rsync_url rsync://${rsync_server}/
     # are we syncing a tarball? (implies detached signature)
     set is_tarball 0
     if {[string range $rsync_dir end-3 end] eq ".tar"} {
         set is_tarball 1
-        set mp_source_path [file join $portdbpath sources $rsync_server [file dirname $rsync_dir]]
+        set tarballs_dir [file dirname $rsync_dir]
+        append rsync_url ${tarballs_dir}/
+        set mp_source_path [file join $portdbpath sources $rsync_server $tarballs_dir]
+        set tarfile [file tail $rsync_dir]
+        set include_options " --include=[macports::shellescape /${tarfile}] --include=[macports::shellescape /${tarfile}.rmd160] --exclude=*"
     } else {
         if {[string index $rsync_dir end] ne "/"} {
             append rsync_dir /
         }
+        append rsync_url $rsync_dir
         set mp_source_path [file join $portdbpath sources $rsync_server $rsync_dir]
+        set include_options ""
     }
     # create the path to the to be downloaded sources if it doesn't exist
     if {![file exists $mp_source_path]} {
@@ -68,31 +75,26 @@ proc selfupdate::main {{optionslist {}} {updatestatusvar {}}} {
 
     # sync the MacPorts sources
     ui_msg "$macports::ui_prefix Updating MacPorts base sources using rsync"
-    try -pass_signal {
-        system "$rsync_path $rsync_options rsync://${rsync_server}/$rsync_dir $mp_source_path"
-    } catch {{*} eCode eMessage} {
-        return -code error "Error synchronizing MacPorts sources: $eMessage"
+    macports_try -pass_signal {
+        system "$rsync_path ${rsync_options}${include_options} [macports::shellescape $rsync_url] [macports::shellescape $mp_source_path]"
+    } on error {eMessage} {
+        error "Error synchronizing MacPorts sources: $eMessage"
     }
 
     if {$is_tarball} {
         # verify signature for tarball
-        global macports::archivefetch_pubkeys
-        try -pass_signal {
-            system "$rsync_path $rsync_options rsync://${rsync_server}/${rsync_dir}.rmd160 $mp_source_path"
-        } catch {{*} eCode eMessage} {
-            return -code error "Error synchronizing MacPorts source signature: $eMessage"
-        }
+        global macports::archivefetch_pubkeys macports::hfscompression
         set openssl [macports::findBinary openssl $macports::autoconf::openssl_path]
-        set tarball ${mp_source_path}/[file tail $rsync_dir]
+        set tarball ${mp_source_path}/${tarfile}
         set signature ${tarball}.rmd160
         set verified 0
         foreach pubkey $macports::archivefetch_pubkeys {
-            try -pass_signal {
+            macports_try -pass_signal {
                 exec $openssl dgst -ripemd160 -verify $pubkey -signature $signature $tarball
                 set verified 1
                 ui_debug "successful verification with key $pubkey"
                 break
-            }  catch {{*} eCode eMessage} {
+            } on error {eMessage} {
                 ui_debug "failed verification with key $pubkey"
                 ui_debug "openssl output: $eMessage"
             }
@@ -101,14 +103,21 @@ proc selfupdate::main {{optionslist {}} {updatestatusvar {}}} {
             return -code error "Failed to verify signature for MacPorts source!"
         }
 
+        if {${macports::hfscompression} && [getuid] == 0 &&
+                ![catch {macports::binaryInPath bsdtar}] &&
+                ![catch {exec bsdtar -x --hfsCompression < /dev/null >& /dev/null}]} {
+            ui_debug "Using bsdtar with HFS+ compression (if valid)"
+            set tar "bsdtar --hfsCompression"
+        } else {
+            set tar [macports::findBinary tar $macports::autoconf::tar_path]
+        }
         # extract tarball and move into place
-        set tar [macports::findBinary tar $macports::autoconf::tar_path]
         file mkdir ${mp_source_path}/tmp
-        set tar_cmd "$tar -C ${mp_source_path}/tmp -xf $tarball"
-        try -pass_signal {
+        set tar_cmd "$tar -C [macports::shellescape ${mp_source_path}/tmp] -xf [macports::shellescape $tarball]"
+        macports_try -pass_signal {
             system $tar_cmd
-        } catch {*} {
-            return -code error "Failed to extract MacPorts sources from tarball!"
+        } on error {} {
+            error "Failed to extract MacPorts sources from tarball!"
         }
         file delete -force ${mp_source_path}/base
         file rename ${mp_source_path}/tmp/base ${mp_source_path}/base
@@ -145,7 +154,7 @@ proc selfupdate::main {{optionslist {}} {updatestatusvar {}}} {
     set comp [vercmp $macports_version_new $macports::autoconf::macports_version]
 
     # syncing ports tree.
-    if {![info exists options(ports_selfupdate_nosync)] || !$options(ports_selfupdate_nosync)} {
+    if {![info exists options(ports_selfupdate_no-sync)] || !$options(ports_selfupdate_no-sync)} {
         if {$comp > 0} {
             # updated portfiles potentially need new base to parse - tell sync to try to
             # use prefabricated PortIndex files and signal if it couldn't
@@ -153,8 +162,8 @@ proc selfupdate::main {{optionslist {}} {updatestatusvar {}}} {
         }
         try {
             mportsync $optionslist
-        }  catch {{*} eCode eMessage} {
-            return -code error "Couldn't sync the ports tree: $eMessage"
+        } on error {eMessage} {
+            error "Couldn't sync the ports tree: $eMessage"
         }
     }
 
@@ -186,25 +195,58 @@ proc selfupdate::main {{optionslist {}} {updatestatusvar {}}} {
                 append configure_args " --with-unsupported-prefix"
             }
 
-            # Choose a sane compiler
+            # Choose a sane compiler and SDK
             set cc_arg {}
+            set sdk_arg {}
+            set jobs [macports::get_parallel_jobs yes]
             if {$::macports::os_platform eq "darwin"} {
-                set cc_arg "CC=/usr/bin/cc OBJC=/usr/bin/cc "
+                catch {exec /usr/bin/cc 2>@1} output
+                set output [join [lrange [split $output "\n"] 0 end-1] "\n"]
+                if {[string match -nocase *license* $output]} {
+                    ui_error "It seems you have not accepted the Xcode license; unable to build."
+                    ui_error "Agree to the license by opening Xcode or running `sudo xcodebuild -license'."
+                    return -code error "Xcode license acceptance required"
+                }
+
+                set cc_arg "CC=/usr/bin/cc "
+                if {$::macports::os_major >= 18 || ![file exists /usr/include/sys/cdefs.h]} {
+                    set cltpath /Library/Developer/CommandLineTools
+                    set sdk_version $::macports::macos_version_major
+                    set check_dirs [list ${cltpath}/SDKs \
+                        ${::macports::developer_dir}/Platforms/MacOSX.platform/Developer/SDKs \
+                        ${::macports::developer_dir}/SDKs]
+                    foreach check_dir $check_dirs {
+                        set sdk ${check_dir}/MacOSX${sdk_version}.sdk
+                        if {[file exists $sdk]} {
+                            set sdk_arg "SDKROOT=[macports::shellescape ${sdk}] "
+                            break
+                        } elseif {$::macports::os_major >= 20} {
+                            set matches [glob -nocomplain -directory ${check_dir} MacOSX${sdk_version}*.sdk]
+                            if {[llength $matches] > 1} {
+                                set matches [lsort -decreasing -command vercmp $matches]
+                            }
+                            if {[llength $matches] > 0} {
+                                set sdk_arg "SDKROOT=[macports::shellescape [lindex $matches 0]] "
+                                break
+                            }
+                        }
+                    }
+                }
             }
 
             # do the actual configure, build and installation of new base
             ui_msg "Installing new MacPorts release in $prefix as ${owner}:${group}; permissions ${perms}\n"
             try {
-                system -W $mp_source_path "${cc_arg}./configure $configure_args && make SELFUPDATING=1 && make install SELFUPDATING=1"
-            } catch {{*} eCode eMessage} {
-                return -code error "Error installing new MacPorts base: $eMessage"
+                system -W $mp_source_path "${cc_arg}${sdk_arg}./configure $configure_args && ${sdk_arg}make -j${jobs} SELFUPDATING=1 && make install SELFUPDATING=1"
+            } on error {eMessage} {
+                error "Error installing new MacPorts base: $eMessage"
             }
             if {[info exists updatestatus]} {
                 set updatestatus yes
             }
         }
     } elseif {$comp < 0} {
-        ui_msg "$macports::ui_prefix MacPorts base is probably trunk or a release candidate"
+        ui_msg "$macports::ui_prefix MacPorts base is probably master or a release candidate"
     } else {
         ui_msg "$macports::ui_prefix MacPorts base is already the latest version"
     }
@@ -214,11 +256,11 @@ proc selfupdate::main {{optionslist {}} {updatestatusvar {}}} {
     ui_debug "Setting MacPorts sources ownership to $sources_owner"
     try {
         exec [macports::findBinary chown $macports::autoconf::chown_path] -R $sources_owner [file join $portdbpath sources/]
-    }  catch {{*} eCode eMessage} {
-        return -code error "Couldn't change permissions of the MacPorts sources at $mp_source_path to ${sources_owner}: $eMessage"
+    } on error {eMessage} {
+        error "Couldn't change permissions of the MacPorts sources at $mp_source_path to ${sources_owner}: $eMessage"
     }
 
-    if {![info exists options(ports_selfupdate_nosync)] || !$options(ports_selfupdate_nosync)} {
+    if {![info exists options(ports_selfupdate_no-sync)] || !$options(ports_selfupdate_no-sync)} {
         if {[info exists needed_portindex]} {
             ui_msg "Not all sources could be fully synced using the old version of MacPorts."
             ui_msg "Please run selfupdate again now that MacPorts base has been updated."
